@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015 Red Hat, Inc.
+ * Copyright (C) 2015,2017 Red Hat, Inc.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,9 @@
 
 #include <sys/types.h>
 #include <errno.h>
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
@@ -1134,6 +1137,15 @@ cm_json_find(struct cm_json *json, const char *path)
 		component = talloc_strndup(json, p, q - p);
 		if (this->type == cm_json_type_object) {
 			this = cm_json_get(this, component);
+			if (this == NULL) {
+				l = strtol(component, &end, 10);
+				if ((end == NULL) || ((*end != '/') && (*end != '\0')) || (l < 0)) {
+					this = NULL;
+					talloc_free(component);
+					continue;
+				}
+				this = cm_json_nth_val(this, l);
+			}
 			talloc_free(component);
 			continue;
 		}
