@@ -204,7 +204,8 @@ main(int argc, const char **argv)
 	int prefer_non_renewal = 0, can_renewal = 0;
 	int response_code = 0, response_code2 = 0;
 	enum known_ops op = op_unset;
-	const char *id = NULL, *cainfo = NULL;
+	char *id = NULL;
+	const char *cainfo = NULL;
 	char *message = NULL, *rekey_message = NULL;
 	const char *mode = NULL, *content_type = NULL, *content_type2 = NULL;
 	void *ctx;
@@ -384,13 +385,16 @@ main(int argc, const char **argv)
 			if ((message == NULL) || (strlen(message) == 0)) {
 				if (poptPeekArg(pctx) != NULL) {
 					message = cm_submit_u_from_file(poptGetArg(pctx));
+					if ((message == NULL) || (strlen(message) == 0)) {
+						printf(_("Error reading request.  Expected PKCS7 data containing a GetInitialCert pkiMessage, got nothing.\n"));
+						rval = CM_SUBMIT_STATUS_NEED_SCEP_MESSAGES;
+						goto done;
+					}
+				} else {
+					rval = CM_SUBMIT_STATUS_NEED_SCEP_MESSAGES;
+					goto done;
 				}
 			}
-			if ((message == NULL) || (strlen(message) == 0)) {
-				printf(_("Error reading request.  Expected PKCS7 data containing a GetInitialCert pkiMessage, got nothing.\n"));
-				rval = CM_SUBMIT_STATUS_NEED_SCEP_MESSAGES;
-				goto done;
-			}
 			/* First step: read capabilities for our use. */
 			params = talloc_asprintf(ctx, "operation=" OP_GET_CA_CAPS "&message=%s", id);
 		}
@@ -404,13 +408,16 @@ main(int argc, const char **argv)
 			if ((message == NULL) || (strlen(message) == 0)) {
 				if (poptPeekArg(pctx) != NULL) {
 					message = cm_submit_u_from_file(poptGetArg(pctx));
+					if ((message == NULL) || (strlen(message) == 0)) {
+						printf(_("Error reading request.  Expected PKCS7 data containing a PKCSReq pkiMessage, got nothing.\n"));
+						rval = CM_SUBMIT_STATUS_NEED_SCEP_MESSAGES;
+						goto done;
+					}
+				} else {
+					rval = CM_SUBMIT_STATUS_NEED_SCEP_MESSAGES;
+					goto done;
 				}
 			}
-			if ((message == NULL) || (strlen(message) == 0)) {
-				printf(_("Error reading request.  Expected PKCS7 data containing a PKCSReq pkiMessage, got nothing.\n"));
-				rval = CM_SUBMIT_STATUS_NEED_SCEP_MESSAGES;
-				goto done;
-			}
 			/* First step: read capabilities for our use. */
 			params = talloc_asprintf(ctx, "operation=" OP_GET_CA_CAPS "&message=%s", id);
 		}
@@ -496,7 +503,7 @@ main(int argc, const char **argv)
 	}
 
 	/* Submit the first request. */
-	hctx = cm_submit_h_init(ctx, "GET", url, params, NULL, NULL,
+	hctx = cm_submit_h_init(ctx, "GET", url, params, NULL, NULL, NULL,
 				cainfo, NULL, NULL, NULL, NULL,
 				cm_submit_h_negotiate_off,
 				cm_submit_h_delegate_off,
@@ -593,7 +600,7 @@ main(int argc, const char **argv)
 	}
 	/* Submit a second HTTP request if we have one to make. */
 	if (params2 != NULL) {
-		hctx = cm_submit_h_init(ctx, "GET", url, params2, NULL, NULL,
+		hctx = cm_submit_h_init(ctx, "GET", url, params2, NULL, NULL, NULL,
 					NULL, NULL, NULL, NULL, NULL,
 					cm_submit_h_negotiate_off,
 					cm_submit_h_delegate_off,
@@ -794,7 +801,7 @@ main(int argc, const char **argv)
 						 OP_GET_CA_CERT
 						 "&message=%d", i++);
 			hctx = cm_submit_h_init(ctx, "GET", url, params,
-						NULL, NULL, NULL, NULL,
+						NULL, NULL, NULL, NULL, NULL,
 						NULL, NULL, NULL,
 						cm_submit_h_negotiate_off,
 						cm_submit_h_delegate_off,
