@@ -1188,7 +1188,7 @@ restart:
 		fprintf(stderr, "url = \"%s%s%s\"\n", uri,
 		        params ? "?" : "", params ? params : "");
 	}
-	hctx = cm_submit_h_init(ctx, method, uri, params, NULL, NULL,
+	hctx = cm_submit_h_init(ctx, method, uri, params, NULL, NULL, NULL,
 				cainfo, capath, sslcert, sslkey, sslpin,
 				cm_submit_h_negotiate_off,
 				cm_submit_h_delegate_off,
