@@ -218,6 +218,44 @@ cm_tdbusm_get_vs(DBusMessage *msg, void *parent, char **s)
 	return -1;
 }
 
+int
+cm_tdbusm_get_vn(DBusMessage *msg, void *parent, long *n)
+{
+	DBusError err;
+	DBusMessageIter iter, sub_iter;
+	int64_t *i64;
+
+	dbus_error_init(&err);
+
+	if (dbus_message_iter_init(msg, &iter) == FALSE) {
+		if (dbus_error_is_set(&err)) {
+			cm_log(3, "DBus error: %s", err.message);
+			dbus_error_free(&err);
+		} else {
+			cm_log(3, "Unknown DBus error.");
+		}
+		return -1;
+	}
+
+	if (dbus_message_iter_get_arg_type(&iter) == DBUS_TYPE_VARIANT) {
+		dbus_message_iter_recurse(&iter, &sub_iter);
+		if (dbus_message_iter_get_arg_type(&sub_iter) == DBUS_TYPE_INT64) {
+			dbus_message_iter_get_basic(&sub_iter, &i64);
+			*n = i64;
+			return 0;
+		}
+	}
+
+	if (dbus_error_is_set(&err)) {
+		cm_log(3, "Failed to extract data from DBus message: %s", err.message);
+		dbus_error_free(&err);
+	} else {
+		cm_log(3, "Failed to extract data from DBus message.");
+	}
+	*n = 0;
+	return -1;
+}
+
 int
 cm_tdbusm_get_s(DBusMessage *msg, void *parent, char **s)
 {
@@ -935,6 +973,105 @@ cm_tdbusm_get_sssnasasasnas(DBusMessage *msg, void *parent,
 	return 0;
 }
 
+int
+cm_tdbusm_get_sssnasasasnasn(DBusMessage *msg, void *parent,
+			    char **s1, char **s2, char **s3, long *n1,
+			    char ***as1, char ***as2, char ***as3,
+			    long *n2, char ***as4, long *n3)
+{
+	DBusError err;
+	char **tmp1, **tmp2, **tmp3, **tmp4;
+	int64_t i641, i642, i643;
+	int32_t i321, i322, i323;
+	int16_t i161, i162, i163;
+	int i, j, k, l;
+	*s1 = NULL;
+	*s2 = NULL;
+	*s3 = NULL;
+	*as1 = NULL;
+	*as2 = NULL;
+	*as3 = NULL;
+	*as4 = NULL;
+	dbus_error_init(&err);
+	if (!dbus_message_get_args(msg, &err,
+				   DBUS_TYPE_STRING, s1,
+				   DBUS_TYPE_STRING, s2,
+				   DBUS_TYPE_STRING, s3,
+				   DBUS_TYPE_INT64, &i641,
+				   DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &tmp1, &i,
+				   DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &tmp2, &j,
+				   DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &tmp3, &k,
+				   DBUS_TYPE_INT64, &i642,
+				   DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &tmp4, &l,
+				   DBUS_TYPE_INT64, &i643,
+				   DBUS_TYPE_INVALID)) {
+		if (dbus_error_is_set(&err)) {
+			dbus_error_free(&err);
+			dbus_error_init(&err);
+		}
+		if (!dbus_message_get_args(msg, &err,
+					   DBUS_TYPE_STRING, s1,
+					   DBUS_TYPE_STRING, s2,
+					   DBUS_TYPE_STRING, s3,
+					   DBUS_TYPE_INT32, &i321,
+					   DBUS_TYPE_ARRAY, DBUS_TYPE_STRING,
+					   &tmp1, &i,
+					   DBUS_TYPE_ARRAY, DBUS_TYPE_STRING,
+					   &tmp2, &j,
+					   DBUS_TYPE_ARRAY, DBUS_TYPE_STRING,
+					   &tmp3, &k,
+					   DBUS_TYPE_INT32, &i322,
+					   DBUS_TYPE_ARRAY, DBUS_TYPE_STRING,
+					   &tmp4, &l,
+					   DBUS_TYPE_INT32, &i323,
+					   DBUS_TYPE_INVALID)) {
+			if (dbus_error_is_set(&err)) {
+				dbus_error_free(&err);
+				dbus_error_init(&err);
+			}
+			if (!dbus_message_get_args(msg, &err,
+						   DBUS_TYPE_STRING, s1,
+						   DBUS_TYPE_STRING, s2,
+						   DBUS_TYPE_STRING, s3,
+						   DBUS_TYPE_INT16, &i161,
+						   DBUS_TYPE_ARRAY,
+						   DBUS_TYPE_STRING, &tmp1, &i,
+						   DBUS_TYPE_ARRAY,
+						   DBUS_TYPE_STRING, &tmp2, &j,
+						   DBUS_TYPE_ARRAY,
+						   DBUS_TYPE_STRING, &tmp3, &k,
+						   DBUS_TYPE_INT16, &i162,
+						   DBUS_TYPE_ARRAY,
+						   DBUS_TYPE_STRING, &tmp4, &l,
+						   DBUS_TYPE_INT16, &i163,
+						   DBUS_TYPE_INVALID)) {
+				if (dbus_error_is_set(&err)) {
+					dbus_error_free(&err);
+					dbus_error_init(&err);
+				}
+				return -1;
+			}
+			i321 = i161;
+			i322 = i162;
+			i323 = i163;
+		}
+		i641 = i321;
+		i642 = i322;
+		i643 = i323;
+	}
+	*s1 = *s1 ? talloc_strdup(parent, *s1) : NULL;
+	*s2 = *s2 ? talloc_strdup(parent, *s2) : NULL;
+	*s3 = *s3 ? talloc_strdup(parent, *s3) : NULL;
+	*n1 = i641;
+	*n2 = i642;
+	*n3 = i643;
+	*as1 = cm_tdbusm_take_dbus_string_array(parent, tmp1, i);
+	*as2 = cm_tdbusm_take_dbus_string_array(parent, tmp2, j);
+	*as3 = cm_tdbusm_take_dbus_string_array(parent, tmp3, k);
+	*as4 = cm_tdbusm_take_dbus_string_array(parent, tmp4, l);
+	return 0;
+}
+
 int
 cm_tdbusm_get_sasasasnas(DBusMessage *msg, void *parent, char **s,
 			 char ***as1, char ***as2, char ***as3,
@@ -1856,6 +1993,57 @@ cm_tdbusm_set_sssnasasasnas(DBusMessage *msg,
 	}
 }
 
+int
+cm_tdbusm_set_sssnasasasnasn(DBusMessage *msg,
+			    const char *s1, const char *s2, const char *s3,
+			    long n1, const char **as1, const char **as2,
+			    const char **as3, long n2, const char **as4,
+			    long n3)
+{
+	int64_t i1 = n1, i2 = n2, i3 = n3;
+	if (s1 == NULL) {
+		s1 = empty_string;
+	}
+	if (s2 == NULL) {
+		s2 = empty_string;
+	}
+	if (s3 == NULL) {
+		s3 = empty_string;
+	}
+	if (as1 == NULL) {
+		as1 = empty_string_array;
+	}
+	if (as2 == NULL) {
+		as2 = empty_string_array;
+	}
+	if (as3 == NULL) {
+		as3 = empty_string_array;
+	}
+	if (as4 == NULL) {
+		as4 = empty_string_array;
+	}
+	if (dbus_message_append_args(msg,
+				     DBUS_TYPE_STRING, &s1,
+				     DBUS_TYPE_STRING, &s2,
+				     DBUS_TYPE_STRING, &s3,
+				     DBUS_TYPE_INT64, &i1,
+				     DBUS_TYPE_ARRAY, DBUS_TYPE_STRING,
+				     &as1, cm_tdbusm_array_length(as1),
+				     DBUS_TYPE_ARRAY, DBUS_TYPE_STRING,
+				     &as2, cm_tdbusm_array_length(as2),
+				     DBUS_TYPE_ARRAY, DBUS_TYPE_STRING,
+				     &as3, cm_tdbusm_array_length(as3),
+				     DBUS_TYPE_INT64, &i2,
+				     DBUS_TYPE_ARRAY, DBUS_TYPE_STRING,
+				     &as4, cm_tdbusm_array_length(as4),
+				     DBUS_TYPE_INT64, &i3,
+				     DBUS_TYPE_INVALID)) {
+		return 0;
+	} else {
+		return -1;
+	}
+}
+
 int
 cm_tdbusm_set_sasasasnas(DBusMessage *msg, const char *s,
 			 const char **as1, const char **as2,
